import React, {
  useImperativeHandle,
  useEffect,
  useRef,
  forwardRef,
} from 'react';
import { type ViewStyle, SafeAreaView } from 'react-native';
import WebView from 'react-native-webview';
import type {
  COMMANDS_FROM_MAP,
  COMMANDS_TO_MAP,
  MAP_EVENT_LISTENER,
  MAP_VIEW_REFERENCE,
} from './CommandTypes';
import type {
  JIBESTREAM_STYLE,
  JIBESTREAM_DESTINATION,
  INSTRUCTION,
  JIBESTREAM_WAYPOINT,
  JIBESTREAM_PATH,
  MAP_CREDENTIALS,
  MAP_COMPASS_PROPERTIES,
  MAP_FLOOR_SELECTOR_PROPERTIES,
  MAP_ZOOM_BUTTON_PROPERTIES,
  MAP_SEARCH_PROPERTIES,
  JIBESTREAM_DESTINATION_SET,
} from './JibestreamTypes';

const MapHtml = require('./JibestreamMap.html');
type LOG_LEVEL_TYPE = 'LOG' | 'DEBUG' | 'INFO' | 'WARN' | 'ERROR';

const handleLogMessage = (level: LOG_LEVEL_TYPE, message: string) => {
  switch (level) {
    case 'LOG':
      console.log('MESSAGE FROM WEBVIEW:', message);
      break;
    case 'DEBUG':
      console.debug('MESSAGE FROM WEBVIEW:', message);
      break;
    case 'INFO':
      console.info('MESSAGE FROM WEBVIEW:', message);
      break;
    case 'WARN':
      console.warn('MESSAGE FROM WEBVIEW:', message);
      break;
    case 'ERROR':
      console.error('MESSAGE FROM WEBVIEW:', message);
      break;
  }
};

const MapView = (
  props: {
    style?: ViewStyle;
    onMapMessage: MAP_EVENT_LISTENER;
  },
  parentRef: React.Ref<unknown> | undefined
) => {
  const webviewRef = useRef(null);

  useEffect(() => {}, []);

  const onWayfindToWaypoint = (waypointId: number) => {
    sendCommandToMap('WAYFIND_TO_WAYPOINT', waypointId);
  };

  const onWayfindBetweenWaypoints = (
    originId: number,
    destinationId: number
  ) => {
    sendCommandToMap('WAYFIND_BETWEEN_WAYPOINTS', { originId, destinationId });
  };

  const drawNavigationPath = (
    pathId: string,
    shouldFocus: boolean = true,
    style: JIBESTREAM_STYLE = {
      stroke: '#ffc956',
      strokeWidth: 3,
    }
  ) => {
    sendCommandToMap('DRAW_NAVIGATION_PATH', { pathId, style, shouldFocus });
  };

  const onStyleShape = (
    unitId: number,
    style: JIBESTREAM_STYLE = { fill: '#ABABAB' }
  ) => {
    sendCommandToMap('STYLE_SHAPE', { id: unitId, style });
  };

  const onStyleUnitsForDestination = (
    destination: JIBESTREAM_DESTINATION,
    style: JIBESTREAM_STYLE = { fill: '#ABABAB' }
  ) => {
    sendCommandToMap('STYLE_SHAPE', { destination, style });
  };

  const onSetMapVenue = (
    credentials: MAP_CREDENTIALS,
    compass?: MAP_COMPASS_PROPERTIES,
    floorSelector?: MAP_FLOOR_SELECTOR_PROPERTIES,
    zoomButtons?: MAP_ZOOM_BUTTON_PROPERTIES,
    search?: MAP_SEARCH_PROPERTIES
  ) => {
    sendCommandToMap('SET_MAP_VENUE', {
      credentials,
      compass,
      floorSelector,
      zoomButtons,
      search,
    });
  };

  useImperativeHandle(parentRef, () => {
    const listeners: MAP_VIEW_REFERENCE = {
      setMapVenue: ({
        credentials,
        compass,
        floorSelector,
        zoomButtons,
        search,
      }: {
        credentials: MAP_CREDENTIALS;
        compass?: MAP_COMPASS_PROPERTIES;
        floorSelector?: MAP_FLOOR_SELECTOR_PROPERTIES;
        zoomButtons?: MAP_ZOOM_BUTTON_PROPERTIES;
        search?: MAP_SEARCH_PROPERTIES;
      }) => {
        onSetMapVenue(credentials, compass, floorSelector, zoomButtons, search);
      },
      centerMapOnPoint: (point: number[]) => {
        sendCommandToMap('CENTER_MAP_ON_POINT', point);
      },
      setFollowBlueDot: (
        enabled: boolean,
        rotate: boolean = false,
        interval: number = 500
      ) => {
        sendCommandToMap('SET_FOLLOW_BLUE_DOT', { enabled, rotate, interval });
      },
      setUserPosition: (lat: number, long: number, heading: number) => {
        sendCommandToMap('UPDATE_USER_LOCATION', { lat, long, heading });
      },
      clearWayfindingPath: () => {
        sendCommandToMap('CLEAR_WAYFINDING_PATH', '');
      },
      wayfindToWaypoint: (waypointId: number) => {
        onWayfindToWaypoint(waypointId);
      },
      wayfindBetweenWaypoints: (originId: number, destinationId: number) => {
        onWayfindBetweenWaypoints(originId, destinationId);
      },
      drawNavigationPath: (
        pathId: string,
        shouldFocus?: boolean,
        style?: JIBESTREAM_STYLE
      ) => {
        drawNavigationPath(pathId, shouldFocus, style);
      },
      styleUnitsForDestination: (
        destination: JIBESTREAM_DESTINATION,
        style?: JIBESTREAM_STYLE
      ) => {
        onStyleUnitsForDestination(destination, style);
      },
      styleShape: (unitId: number, style?: JIBESTREAM_STYLE) => {
        onStyleShape(unitId, style);
      },
    };
    return listeners;
  });

  function onMessage(message: { nativeEvent: { data: string } }) {
    const obj: { command: COMMANDS_FROM_MAP; data: Object } = JSON.parse(
      message.nativeEvent.data
    );
    const command = obj.command;
    const data = obj.data;
    // props.onMapMessage(command, data);
    switch (command) {
      case 'DESTINATION_SELECTED':
        props.onMapMessage.onDestinationSelected?.(
          data as JIBESTREAM_DESTINATION
        );
        break;
      case 'NAVIGATION_PATH_GENERATED':
        const obj = data as {
          pathId: string;
          instructions: INSTRUCTION[];
          start: JIBESTREAM_WAYPOINT;
          end: JIBESTREAM_WAYPOINT;
          path: JIBESTREAM_PATH[];
        };
        props.onMapMessage.onNavigationPathGenerated?.(
          obj.pathId,
          obj.instructions,
          obj.start,
          obj.end,
          obj.path
        );
        break;
      case 'USER_POSITION_UPDATED':
        props.onMapMessage.onUserPositionUpdated?.(data as number[]);
        break;
      case 'WINDOW_LOADED':
        props.onMapMessage.onWindowLoaded?.();
        break;
      case 'GENERATED_SEARCH_DESTINATIONS':
        props.onMapMessage.onSearchDestinationsGenerated?.(
          data as JIBESTREAM_DESTINATION_SET[]
        );
        break;
      case 'CONSOLE_LOG':
        console.log("Received COnsole Log Message");
        const logObj = data as { level: LOG_LEVEL_TYPE; message: string };
        handleLogMessage(logObj.level, logObj.message);
        break;
      default:
        console.log('Unhandled command from map:', command, data);
        break;
    }
  }

  function sendCommandToMap(command: COMMANDS_TO_MAP, data: Object) {
    const clientResponseCode = `
    window.postMessage(${JSON.stringify({ command, data })}, "*");
    true;
  `;
    if (webviewRef.current)
      // @ts-ignore
      webviewRef.current.injectJavaScript(clientResponseCode);
  }

  return (
    <SafeAreaView style={{ flex: 1, width: '100%', height: '100%' }}>
      <WebView
        // injectedJavaScript={debugging}
        ref={webviewRef}
        scalesPageToFit={false}
        mixedContentMode="compatibility"
        onMessage={onMessage}
        source={MapHtml}
      />
    </SafeAreaView>
  );
};

export default forwardRef(MapView);
