import React, { forwardRef, useEffect } from 'react';
import { useImperativeHandle, useRef } from 'react';
import {
  requireNativeComponent,
  UIManager,
  findNodeHandle,
  DeviceEventEmitter,
  NativeEventEmitter,
  NativeModules,
  Platform,
} from 'react-native';
import { ODP_EVENT_LISTENER, ODP_VIEW_REFERENCE } from './CommandTypes';
import { ODP_ON_READY } from './schemas/OdpOnReadyType';
import { ODP_ON_FACILITY_UPDATE } from './schemas/OdpOnFacilityUpdateType';
import { ODP_ON_KEY_VALIDATION } from './schemas/OdpOnKeyValidationType';
import { ODP_ON_NAVIGATION_UPDATE } from './schemas/OdpOnNavigationUpdateType';

const OdpViewManagerCommands = {
  start: 1,
  upload_logs: 2,
};

const startOdp = (viewId: number | null, key: string, host: string) =>
  UIManager.dispatchViewManagerCommand(
    viewId,
    OdpViewManagerCommands.start.toString(),
    [key, host]
  );

const uploadLogs = (viewId: number | null) =>
  UIManager.dispatchViewManagerCommand(
    viewId,
    OdpViewManagerCommands.upload_logs.toString(),
    []
  );

// const LINKING_ERROR =
//   `The package 'react-native-library' doesn't seem to be linked. Make sure: \n\n` +
//   Platform.select({ ios: "- You have run 'pod install'\n", default: '' }) +
//   '- You rebuilt the app after installing the package\n' +
//   '- You are not using Expo Go\n';

const OdpViewManager = requireNativeComponent('OdpViewManager');

// const OdpComponent =
//   UIManager.getViewManagerConfig('OdpViewManager') != null
//     ? requireNativeComponent('OdpViewManager')
//     : () => {
//         throw new Error(LINKING_ERROR);
//       };

const { CTPEventEmitter, OdpViewManagerManager } = NativeModules;

const OdpModule = (
  props: {
    onOdpEvent: ODP_EVENT_LISTENER;
  },
  parentRef: React.Ref<unknown> | undefined
) => {
  const nodeRef = useRef(null);

  useEffect(() => {
    console.log('Initializing Event Emitter');
    let eventEmitter: typeof DeviceEventEmitter | NativeEventEmitter | null =
      null;
    if (Platform.OS == 'android') {
      eventEmitter = DeviceEventEmitter;
    } else if (Platform.OS == 'ios') {
      eventEmitter = new NativeEventEmitter(CTPEventEmitter);
    }
    eventEmitter?.addListener('onReady', (event) => {
      console.log('ODP onReady');
      props.onOdpEvent.onReady?.(event as ODP_ON_READY);
    });
    eventEmitter?.addListener('onKeyValidation', (event) => {
      console.log('ODP onKeyValidation');
      props.onOdpEvent.onKeyValidation?.(event as ODP_ON_KEY_VALIDATION);
    });
    eventEmitter?.addListener('onNavigationUpdate', (event) => {
      props.onOdpEvent.onNavigationUpdate?.(event as ODP_ON_NAVIGATION_UPDATE);
    });
    eventEmitter?.addListener('onFacilityUpdate', (event) => {
      console.log('ODP onFacilityUpdate');
      props.onOdpEvent.onFacilityUpdate?.(event as ODP_ON_FACILITY_UPDATE);
    });
    return () => {
      console.log('Removing Event Emitters');
      eventEmitter?.removeAllListeners('onReady');
      eventEmitter?.removeAllListeners('onKeyValidation');
      eventEmitter?.removeAllListeners('onNavigationUpdate');
      eventEmitter?.removeAllListeners('onFacilityUpdate');
    };
  }, []);

  useImperativeHandle(parentRef, () => {
    const nodeHandle = findNodeHandle(nodeRef.current);

    const listeners: ODP_VIEW_REFERENCE = {
      startOdp: (key: string, host: string) => {
        if (Platform.OS == 'ios') {
          // OdpComponent.start(key, host);
          UIManager.getViewManagerConfig('OdpViewManager').Commands.start;
          OdpViewManagerManager.start(key, host);
        } else if (Platform.OS == 'android') {
          startOdp(nodeHandle, key, host);
        }
      },
      uploadLogs: () => {
        if (Platform.OS == 'android') {
          uploadLogs(nodeHandle);
        }
      },
    };

    return listeners;
  });

  return <OdpViewManager ref={nodeRef}></OdpViewManager>;
};

export default forwardRef(OdpModule);
